$.fn.countingZone = function(options) {
    var self = this;
    var defaults = {
        view_only: false,
        restriction: {
            left: 0,
            right: 1,
            top: 0,
            bottom: 1,
            min_width: 0,
            min_height: 0.05,
            max_width: 1,
            max_height: 1
        }
    };

    var zone = JSON.parse(JSON.stringify(params.counter.zone));
    var installation_height = JSON.parse(JSON.stringify(params.meta.height));

    // For convenience
    var r = options.restriction;
    var z = zone;

    function res(v) {
        return (v * 1000)|0;
    }

    function unres(v) {
        return v / 1000;
    }

    function addSlider(title, name, opts) {
        var slide_callback = opts.slide,
            $ctrl = $("<div/>").addClass("ctrlHolder"),
            $slider;
        $("<label/>").text(title).appendTo($ctrl);
        /*
        opts.slide = function(cb) {
            var valid = slide_callback.apply(this, arguments);
            if ( opts.range ) {
                var v = $slider.slider("values");
                $input.val(JSON.stringify([unres(v[0]), unres(v[1])]));
            } else {
                $input.val(JSON.stringify(unres($slider.slider("value"))));
            }
            return valid;
        };
        */
        if ( !options.view_only ) {
            $slider = $("<div/>").slider(opts||{}).addClass("uniform-slider").appendTo($ctrl);
            $ctrl.appendTo(self);
        }
        return $slider;
    }

    options = $.extend({}, defaults, options);

    var width = options.width || 100;
    var height = options.height || 100;

    var canvas = new fabric.StaticCanvas(options.canvas, {
        width: width,
        height: height
    });


    /*
     * This variable defines all objects that are rendered on the canvas.
     * Their position on the canvas is set with the updatePositions function.
     */
    var objs = {
        mask_top: new fabric.Rect({
            opacity: 0.45,
            width: width
        }),
        mask_left: new fabric.Rect({
            opacity: 0.45,
        }),
        mask_right: new fabric.Rect({
            opacity: 0.45,
            width: width
        }),
        mask_bottom: new fabric.Rect({
            opacity: 0.45,
            width: width
        }),
        boxline_topleft: new fabric.Line([],
            {
                strokeWidth:2,
                selectable:false
            }
        ),
        boxline_topright: new fabric.Line([],
            {
                strokeWidth:2,
                selectable:false
            }
        ),
        boxline_bottomleft: new fabric.Line([],
            {
                strokeWidth:2,
                selectable:false
            }
        ),
        boxline_bottomright: new fabric.Line([],
            {
                strokeWidth:2,
                selectable:false
            }
        ),
        boxline_left: new fabric.Line([],
            {
                stroke:'rgba(227,0,15, 0.5)',
                strokeWidth:2,
                selectable:false
            }
        ),
        boxline_right: new fabric.Line([],
            {
                stroke:'rgba(227,0,15, 0.5)',
                strokeWidth:2,
                selectable:false
            }
        ),
        active: new fabric.Rect({
            opacity: 0.5,
            fill: "rgba(255, 0, 0, 0.5)",
            stroke : 'rgba(227,0,15, 0.25)',
            strokeWidth : 2
        }),
        dir_in_text: new fabric.Text("Direction In",{
            fontFamily:"Arial",
            fontSize:20,
            opacity:0,
            fill:"#FFF",
            shadow: 'rgba(0,0,0,1) 1px 1px',
            left:5,
            top:5,
        }),
        dir_in_up: new fabric.Text("\uf062",{
            fontFamily: 'FontAwesome',
            fontSize:20,
            opacity:0,
            fill:"#FFF",
            shadow: 'rgba(0,0,0,1) 1px 1px',
            selectable: false,
        }),
        dir_in_down: new fabric.Text("\uf063",{
            fontFamily: 'FontAwesome',
            fontSize:20,
            opacity:0,
            fill:"#FFF",
            shadow: 'rgba(0,0,0,1) 1px 1px',
            selectable: false,
        }),
    };

    function sanitizePosition() {
        var r = options.restriction;
        zone.left = Math.max(zone.left, r.left);
        zone.top = Math.max(zone.top, r.top);
        zone.right = Math.min(zone.right, r.right);
        zone.bottom = Math.min(zone.bottom, r.bottom);
        if ( zone.right - zone.left < r.min_width) {
            zone.right += r.min_width - (zone.right - zone.left);
        }
        if ( zone.right - zone.left > r.max_width) {
            zone.right -= (zone.right - zone.left) - r.max_width;
        }
        if ( zone.right > r.right ) {
            zone.left -= zone.right - r.right;
            zone.right = r.right;
        }
        if ( zone.left < r.left ) {
            zone.left = r.left;
        }
    }

    function updateEverything(forced_crop) {
        position_arrows();
        sanitizePosition();
        updatePositions(forced_crop);
    }

    function redraw(forced_crop) {
        // Forced crop likely not needed, but could be used when getting params from backend.
        // Since this might be called from inside the slide-callback
        // we must delay the rendering, so the value is updated.
        setTimeout(function() {
            updateEverything(forced_crop);
            canvas.renderAll();
            $("#zone_json").val(JSON.stringify(zone));
            // console.log("new zone val: ", $("#zone_json").val());
        }, 0);
    }

    for(var i in objs) {
        canvas.add(objs[i]);
    }

    // Needed so initial values on sliders are correct.
    sanitizePosition();

    var $offset, $height, $width;
    function setup_dom(reinitialization) {
        if (typeof reinitialization != "undefined" && reinitialization===true) {
            /* The reinitialization might not be needed if sliders can be set correctly on backend call for new zone"*/
            self.find(".ui-slider").parent(".ctrlHolder").remove();
        }
        $offset = addSlider("Counting zone offset", "zone_offset", {
            min: res(r.top + r.min_height/2),
            max: res(r.bottom - r.min_height/2),
            value: res(z.top + z.bottom) / 2,
            slide: function(event, ui) {
                var $this = $(this);
                var old = $this.slider("value");
                var height = unres($height.slider("value"));
                var range = r.bottom - r.top - height;
                var new_top = unres(ui.value) - height/2;
                var new_bottom = unres(ui.value) + height/2;
                var valid_move = new_top >= r.top && new_bottom <= r.bottom;

                if (!valid_move) {
                    if ( new_top < r.top ) {
                        $this.slider("value", res(r.top) + $height.slider("value")/2);
                    } else {
                        $this.slider("value", res(r.bottom) - $height.slider("value")/2);
                    }
                    ui.value = $this.slider("value");
                    new_top = unres(ui.value) - height/2;
                    new_bottom = unres(ui.value) + height/2;
                }

                z.top = new_top;
                z.bottom = new_bottom;

                redraw();
                return valid_move;
            }
        });

        $height = addSlider("Counting zone height", "zone_height", {
            min: res(r.min_height),
            max: res(r.max_height),
            value: [res(z.bottom) - res(z.top)],
            slide: function (event, ui) {
                var $this = $(this);
                var old  = $this.slider("value");
                var c = (z.bottom + z.top) / 2;

                z.bottom = c + unres(ui.value) / 2;
                z.top = c - unres(ui.value) / 2;

                if ( z.bottom > r.bottom ) {
                    z.top -= z.bottom - r.bottom;
                    z.bottom = r.bottom;
                }

                if ( z.top < r.top ) {
                    z.bottom += r.top - z.top;
                    z.top = r.top;
                }
                c = (z.bottom + z.top) / 2;
                if ($offset.slider("value") != res(c)) {
                    $offset.slider("value", res(c));
                }
                var height = (z.bottom - z.top) / 2;

                redraw();
            }
        });
        if(r.fix_height) {
            $height.parent().hide();
            $("#zone_height_desc").hide();
        }
        $width = addSlider("Counting zone width", "zone_width", {
            min: res(r.left),
            max: res(r.right),
            values: [res(z.left), res(z.right)],
            range: true,
            slide: function(event, ui){
                var $this = $(this);
                var old = $(this).slider("values");
                var vals = ui.values;
                var left = unres(vals[0]);
                var right = unres(vals[1]);
                var width = unres(vals[1] - vals[0]);
                if ( width < r.min_width){
                    if ( old[0] == vals[0] ) {
                        if ( old[0] > res(r.left) ) {
                            $this.slider("values", [old[0] + vals[1] -  old[1], vals[1]]);
                        } else {
                            $this.slider("values", [res(r.left), res(r.left + r.min_width)]);
                        }
                    } else {
                        if ( old[1] < res(r.right) ) {
                            $this.slider("values", [vals[0], old[1] + vals[0] -  old[0]]);
                        } else {
                            $this.slider("values", [res(r.right - r.min_width), res(r.right)]);
                        }
                    }
                    z.left = unres($width.slider("values")[0]);
                    z.right = unres($width.slider("values")[1]);
                    redraw();
                    return false;
                }
                if ( width > r.max_width) {
                    if ( old[0] == vals[0] ) {
                        $this.slider("values", [old[0] + vals[1] -  old[1], vals[1]]);
                    } else {
                        $this.slider("values", [vals[0], old[1] + vals[0] -  old[0]]);
                    }
                    z.left = unres($width.slider("values")[0]);
                    z.right = unres($width.slider("values")[1]);
                    redraw();
                    return false;
                }
                z.left = unres($width.slider("values")[0]);
                z.right = unres($width.slider("values")[1]);
                redraw();
            }
        });
    }
    setup_dom();

    function calculate_crop(zone) {
        /*
         * Calculates crop box from zone parameters.
         */
        crop =  $.extend({}, zone); //Clone zone to crop
        crop.left = 0.5 - (0.5 - crop.left) * installation_height / (installation_height - 140);
        crop.right = 0.5 + (crop.right - 0.5) * installation_height / (installation_height - 140);
        crop.top = 0.5 - (0.5 - crop.top) * installation_height / (installation_height - 140);
        crop.bottom = 0.5 + (crop.bottom - 0.5) * installation_height / (installation_height - 140);
        return crop;
    }

    function updatePositions(forced_crop) {
        /*This needs row ~430-470 working to make sense.
        if ($meta_height.length !== 0) {
            installation_height = $meta_height.val();
        }*/
        if (typeof forced_crop != "undefined") {
            crop = forced_crop;
            // console.log("forced crop", crop);
        } else {
            crop = calculate_crop(z);
            // console.log("calculated crop:", crop);
        }
        objs.active.left = z.left * width;
        objs.active.top = z.top * height;
        objs.active.height = (z.bottom - z.top) * height;
        objs.active.width = (z.right - z.left) * width;
        objs.mask_top.height = height * crop.top;
        objs.mask_bottom.height = height * (1 - crop.bottom);
        objs.mask_bottom.top = crop.bottom * height;
        objs.mask_left.top = objs.mask_top.height;
        objs.mask_left.height = height * (crop.bottom - crop.top);
        objs.mask_left.width = crop.left * width;
        objs.mask_right.top = objs.mask_top.height;
        objs.mask_right.height = objs.mask_left.height;
        objs.mask_right.width = (1 - crop.right) * width;
        objs.mask_right.left = crop.right * width;
        objs.boxline_topleft.set({
            'x1': objs.active.left,
            'y1': objs.active.top,
            'x2': objs.mask_left.width - 1,
            'y2': objs.mask_top.height - 1
        }).setGradient('stroke', {
            'x1': objs.boxline_bottomright.width / 2,
            'y1': 0,
            'x2': -objs.boxline_bottomright.width / 2,
            'y2': 0,
            'colorStops': {
                0: "rgba(227,0,1,0.45)",
                1: "rgba(0,0,0,0.45)"
            }
        });
        objs.boxline_topright.set({
            'x1': objs.active.left + objs.active.width,
            'y1': objs.active.top,
            'x2': width - objs.mask_right.width + 1,
            'y2': objs.mask_top.height + 1
        }).setGradient('stroke', {
            'x1': -objs.boxline_bottomright.width / 2,
            'y1': 0,
            'x2': objs.boxline_bottomright.width / 2,
            'y2': 0,
            'colorStops': {
                0: "rgba(227,0,1,0.45)",
                1: "rgba(0,0,0,0.45)"
            }
        });
        objs.boxline_bottomleft.set({
            'x1': objs.active.left,
            'y1': objs.active.top + objs.active.height,
            'x2': objs.mask_left.width - 1,
            'y2': objs.mask_bottom.top - 1
        }).setGradient('stroke', {
            'x1': objs.boxline_bottomright.width / 2,
            'y1': 0,
            'x2': -objs.boxline_bottomright.width / 2,
            'y2': 0,
            'colorStops': {
                0: "rgba(227,0,1,0.45)",
                1: "rgba(0,0,0,0.45)"
            }
        });
        objs.boxline_bottomright.set({
            'x1': objs.active.left + objs.active.width,
            'y1': objs.active.top + objs.active.height,
            'x2': width - objs.mask_right.width + 1,
            'y2': objs.mask_bottom.top +1 ,
        }).setGradient('stroke', {
            'x1': -objs.boxline_bottomright.width / 2,
            'y1': 0,
            'x2': objs.boxline_bottomright.width / 2,
            'y2': 0,
            'colorStops': {
                0: "rgba(227,0,1,0.45)",
                1: "rgba(0,0,0,0.45)"
            }
        });
        objs.boxline_left.set({
            'x1': objs.active.left,
            'y1': objs.active.top,
            'x2': objs.active.left,
            'y2': objs.active.top + objs.active.height,
        });
        objs.boxline_right.set({
            'x1': objs.active.left + objs.active.width,
            'y1': objs.active.top,
            'x2': objs.active.left + objs.active.width,
            'y2': objs.active.top + objs.active.height,
        });
    }

    // Trigger slide on all sliders.
    function trigger_slides() {
        [$width, $offset, $height].forEach(function(slider) {
            var $slider = $(slider),
                ui = {};

            ui.value = $slider.slider("value");
            ui.values = $slider.slider("values");
            /*
            ui.value = res(Math.random());
            ui.values = [res(0.2 + 0.6 * Math.random()), res(0.8)];
            */
            $slider.slider("option", "slide").call($slider, {}, ui); // Always a bit of magic needed to dynamically set sliders. This was found in StackOverflow if i'm not mistaken.
        });
    }

    function animate_opacity(object, val, time) {
        var cur_val = object.opacity,
            step = Math.abs(cur_val - val) / 10,
            sleep = time / 10;
        if (typeof object.animation_interval != "undefined") {
            clearInterval(object.animation_interval);
        }
        object.animation_interval = setInterval(function () {
            cur_val = object.opacity;
            if (cur_val > val) {
                object.opacity = cur_val - step;
            } else if (cur_val < val) {
                object.opacity = cur_val + step;
            } else {
                clearInterval(object.animation_interval);
                object.animation_interval = null;
                return;
            }
            if (Math.abs(val - cur_val) < 0.001) {
                clearInterval(object.animation_interval);
                object.animation_interval = null;
                object.opacity = val;
            }
            canvas.renderAll();
        }, sleep);
    }

    function position_arrows() {
        var arrow_left = objs.dir_in_text.left + objs.dir_in_text.width + 5,
            arrow_top = objs.dir_in_text.top;
        objs.dir_in_up.left = arrow_left;
        objs.dir_in_down.left = arrow_left;
        objs.dir_in_up.top = arrow_top;
        objs.dir_in_down.top = arrow_top;
    }

    function show_dir(val) {
        var arrow;
        if (val==="up") {
            arrow = objs.dir_in_up;
        } else {
            arrow = objs.dir_in_down;
        }
        animate_opacity(objs.dir_in_text, 1, 100);
        animate_opacity(arrow, 1, 100);
        redraw();
    }

    function hide_dir(val) {
        if(!$("#direction_in").is(":focus")) {
            var arrow;
            if (val==="up") {
                arrow = objs.dir_in_up;
                objs.dir_in_down.opacity = 0;
            } else {
                arrow = objs.dir_in_down;
                objs.dir_in_up.opacity = 0;
            }
            animate_opacity(objs.dir_in_text, 0, 100);
            animate_opacity(arrow, 0, 100);
            redraw();
        }
    }
    if ( !options.view_only ) {
        /*
         * View only is set on liveview for example, then this does not need to be run.
         */
        trigger_slides();
        $("#direction_in").parents(".ctrlHolder").on(
            {
                mouseenter: function() {
                    show_dir($("#direction_in").val());
                },
                mouseleave: function() {
                    hide_dir($("#direction_in").val());
                }
        });
        $("#direction_in").on({
            focus: function () {
                show_dir($(this).val());
            },
            blur: function() {
                hide_dir($(this).val());
            },
            change: function() {
                var cur_val = $(this).val();
                if (cur_val==="up") {
                    objs.dir_in_up.opacity = 1;
                    objs.dir_in_down.opacity = 0;
                } else {
                    objs.dir_in_up.opacity = 0;
                    objs.dir_in_down.opacity = 1;
                }
                redraw();
            }
        });
    }
    /*
     * The following does not work correctly
     *
    $meta_height = $("#meta_height");
    if ($meta_height.length !== 0) {
        $meta_height.change(function() {
            //When height changes we need to do a full backend recalculation of zone.
            zone = $("#zone_json").val();
            installation_height = $(this).val();
            console.log("old zone:", zone);
            $.ajax({
                url : "full_zone_params_recalculation.json",
                type: "POST",
                contentType : "application/json",
                data : JSON.stringify({"zone": zone, "height": installation_height}),
                dataType : "json",
                success : function (data, textStatus, jqXHR) {
                    // console.log("response from api (crop)", data.crop);
                    // console.log("response from api (zone)", data.zone);
                    zone = data.zone;
                    z = zone;
                    setup_dom(reinitialization=true); // Tried to see if it would help to remove and setup sliders again.
                    sanitizePosition();
                    // redraw(data.crop);
                    redraw();  // probably good enough to redraw without giving the crop calculated in the backend.
                },
                error : function(jqXHR, textStatus, errorThrown) {
                    console.log(textStatus);
                    console.log(errorThrown);
                },
            });
        });
    }*/
    redraw();
    if ( options.view_only ) {
        redraw(); //Everything doesn't render otherwise.. unclear why...
    }
};
