/*global $*/
"use strict";
$.fn.schedule = function (options) {
    /*
     * Base options and variable declarations.
     */
    var self = this,
        defaults = {
            min: 0,
            max: 24 * 12, //Whole day is 24 * 12 five minute steps. -1 To end on 23:59 rather than 24:00
            start: "00:00",
            end: "23:59",
            range: true,
        },
        simple_mode = true,
        $mode_select = $('input[type=radio][name=schedule_type]'),
        $visible_textfields = [],
        $sliders_wrapper = $("#schedule_sliders"),
        $schedule_info = $("#schedule_info"),
        $form = $sliders_wrapper.parents("form"),
        $link_slider = $("#schedule_link_weekdays"),
        $schedule_definition = $('input[type=radio][name=schedule_definition]'),
        sliders_linked = !$link_slider.attr("checked"),
        $simple_elements = $sliders_wrapper.add($link_slider.parents(".ctrlHolder")),
        $advanced_elements = $schedule_definition.parents(".ctrlHolder"); //This is extended with visible textfields during init.
    /*
     * Helper functions
     */
    Number.prototype.toHHMM = function () {
        var minutes_since_midnight = this * 5,
            hours   = Math.floor(minutes_since_midnight / 60),
            minutes = minutes_since_midnight - (hours * 60);
        if (hours === 24) {
            hours = 23;
            minutes = 59;
        }
        if (hours   < 10) {hours   = "0" + hours; }
        if (minutes < 10) {minutes = "0" + minutes; }
        return hours + ':' + minutes;
    };
    String.prototype.to5MinutePeriodsSinceMidnight = function () {
        var hours_and_minutes   = this.split(":"),
            hours = parseInt(hours_and_minutes[0], 10),
            minutes = parseInt(hours_and_minutes[1], 10),
            minutes_since_midnight = hours * 60 + minutes;
        return minutes_since_midnight / 5;
    };
    function validateTime(time) {
        // Time is of the form xx:yy
        var hours_and_minutes = time.split(":");
        if(hours_and_minutes.length === 1){
            return false;
        }
        var hours = parseInt(hours_and_minutes[0],10);
        var minutes = parseInt(hours_and_minutes[1],10);
        return (hours > 23 || minutes > 59) ? false : true;
    }
    function updateTooltips(values, $slider) {
        var curValues = values || [options.min, options.max], // current value (when sliding) or initial value (at start)
            $tooltipOpens = $slider.children('.ui-slider-handle').first().find(".tooltip-inner"),
            $tooltipCloses = $slider.children('.ui-slider-handle').last().find(".tooltip-inner"),
            opens = curValues[0].toHHMM(),
            closes = curValues[1].toHHMM();
        $tooltipOpens.html(opens);
        $tooltipCloses.html(closes);
    }
    function addSlider(title, opts) {
        //var slide_callback = opts.slide,
        var $ctrl = $("<div/>").addClass("ctrlHolder"),
            $slider;
        $("<label/>").text(title).appendTo($ctrl);
        $slider = $("<div/>").slider(opts || {}).addClass("uniform-slider scheduleSlider").appendTo($ctrl);
        $ctrl.appendTo(self);
        return $slider;
    }
    function parseWeekdayValTo5MinutePeriods(weekday_val) {
        var check_advanced_mode =  weekday_val.split(",");
        if(check_advanced_mode.length > 1){
            return false;
        }
        var values = weekday_val.split("-");
        if (values.length < 2 || values.length > 3 || values[0].indexOf(':') === -1 || values[1].indexOf(':') === -1) {
            return false;
        }
        values[0] = values[0].to5MinutePeriodsSinceMidnight();
        values[1] = values[1].to5MinutePeriodsSinceMidnight();
        return [values[0], values[1], values[2] == 'disabled'];
    }
    function switchMode(mode, init) {
        if (mode === "simple") {
            simple_mode = true;
            if (init) {
                $advanced_elements.hide();
                $simple_elements.show();
            } else {
                $schedule_definition.val(["enabled"]); //[] is important here.
                $advanced_elements.slideUp();
                $simple_elements.slideDown();
            }
            if ( mode === 'simple' ) {
                $('#schedule-fieldset .disableCheckbox').trigger('change');
            }
            $schedule_info
                .html("Drag sliders to adjust the schedule for when counter should be <em>enabled</em>.");
        } else if (mode === "advanced") {
            simple_mode = false;
            if (init) {
                $simple_elements.hide();
                $advanced_elements.show();
            } else {
                $simple_elements.slideUp();
                $advanced_elements.slideDown();
            }
            $schedule_info
                .html("Specify the schedule period in the form HH:MM-HH:MM. \
                       For more than one schedule period per day separate opening hours with a comma."
                );
        } else {
            throw "unknown mode in switchMode";
        }
    }
    /*
     * Initialization
     */
    options = $.extend({}, defaults, options);
    $.each(["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"], function (i, day) {
        var $visible_textfield = $("#weekday_" + i + "_advanced"),
            $hidden_textfield = $("#weekday_" + i + "_hidden"),
            weekday_val = $hidden_textfield.val(),
            simple_values,
            $slider;
        var $enable = $("<input type='checkbox'>");
        $slider = addSlider(day, {
            min: options.min,
            max: options.max,
            values: [options.min, options.max],
            range: options.range,
            create: function(event, ui) {
                var $tooltip = $("<div/>").addClass("tooltip")
                        .append($("<div/>").addClass("tooltip-inner"))
                        .append($("<div/>").addClass("tooltip-arrow")),
                    $slider = $(this),
                    $opens = $slider.children('.ui-slider-handle').first(),
                    $closes = $slider.children('.ui-slider-handle').last();
                $opens.append($tooltip.clone().hide()); //attach tooltip to the slider handle
                $closes.append($tooltip.clone().hide()); //attach tooltip to the slider handle
                $opens.add($closes)
                    .focus(function () {
                        $(this).children(".tooltip").show();
                    }).blur(function () {
                        $(this).children(".tooltip").hide();
                    });
                updateTooltips(ui.values, $slider);
            },
            slide: function (event, ui) {
                var old_values = $(this).slider("values"),
                    opens = ui.values[0],
                    closes = ui.values[1];
                if (closes <= opens) {
                    return false;
                }
                updateTooltips(ui.values, $(this));
            },
            change: function (event, ui) {
                if ( !ui ) {
                    ui = {
                        'values': $(this).slider('values')
                    }
                    event = {
                        originalEvent: true
                    }
                }
                var $scheduleLabel = $slider.siblings("label").children(".scheduleLabel"),
                    opens = ui.values[0],
                    closes = ui.values[1],
                    enabled = $enable.is(':checked');
                if (enabled) {
                    $scheduleLabel.html("[" + opens.toHHMM() + "-" + closes.toHHMM() + "]");
                    if (sliders_linked && event.originalEvent) {
                        $sliders_wrapper.find(".ui-slider").each(function () {
                            $(this).slider("values", 0, opens);
                            $(this).slider("values", 1, closes);
                            updateTooltips(ui.values, $(this));
                        });
                    }
                    $visible_textfield.add($hidden_textfield).val(opens.toHHMM() + "-" + closes.toHHMM());
                } else {
                    $scheduleLabel.html("[day disabled]");
                    $visible_textfield.val("");
                }
                prepareSchedule();
            }
        });
        $visible_textfields.push($visible_textfield);
        $.each($visible_textfields, function (i, $textfield) {
            $advanced_elements = $advanced_elements.add($textfield.parents(".ctrlHolder"));
        });
        $slider.siblings("label").append($("<span/>").addClass("scheduleLabel"));
        $visible_textfield.val(weekday_val);
        simple_values = parseWeekdayValTo5MinutePeriods(weekday_val);

        if (simple_values && $('input[type=radio][name=schedule_definition]:checked').val() === "enabled") {
            $enable.prop('checked', !simple_values[2]).trigger('change');
            $slider.slider("values", 0, simple_values[0]);
            $slider.slider("values", 1, simple_values[1]);
            updateTooltips(simple_values, $slider);
        } else {
            simple_mode = false;
        }
        $enable.insertAfter($slider).addClass("disableCheckbox").on('change', function() {
            var enabled = $(this).is(':checked');
            if ( enabled ) {
                $slider.slider('enable');
            } else {
                $slider.slider('disable');
            }
            $slider.slider('option', 'change').call($slider);
        });
    });

    if (simple_mode) {
        switchMode("simple", true);
        $("#schedule_type_simple").prop("checked", true);
    } else {
        switchMode("advanced", true);
        $("#schedule_type_advanced").prop("checked", true);
    }
    $(".weekday_field").validator(function(cb) {
        var val = $(this).val(),
            values = [],
            last_end = -1;
        if (val.indexOf(",") !== -1) {
            values = val.split(",");
        } else {
            values[0] = val;
        }
        $.each(values, function (i, val) {
            var start,
                end,
                interval,
                failed_parse_val = false;
            val = $.trim(val);
            if (val === "") {
                cb();
            } else {
                try {
                    interval = val.split("-");
                    start = interval[0].to5MinutePeriodsSinceMidnight();
                    end =   interval[1].to5MinutePeriodsSinceMidnight();
                    if(validateTime(interval[0]) === false || validateTime(interval[1]) === false  ){
                        cb("Invalid time range entered. Please specify the range in 24 hour format.");
                    }
                } catch (error) {
                    failed_parse_val = true;
                }
                if (failed_parse_val ||
                        !(val.match(/^([0-9]{2}:[0-9]{2})-([0-9]{2}:[0-9]{2});?(;([0-9]{2}:[0-9]{2})-([0-9]{2}:[0-9]{2}))*$/))) {
                    cb("Schedule must be a comma separated list of one or more periods in the form HH:MM-HH:MM. ");
                }
                if (start >= end) {
                    cb("Schedule period start time must be less than end time.");
                }
                if (last_end >= start) {
                    cb("Schedule period start time must be after previous period end time.");
                }
                last_end = end;
            }
        });
        prepareSchedule();
        cb();
    });
    /*
     * Event handling
     */
    $mode_select.change(function () {
        if ( !simple_mode ) {
            if ( !confirm('Changing from advanced to simple scheduling mode might alter your current schedule, even if you switch back. Are you sure?') ) {
                $mode_select.not(this).prop('checked', true);
                return false;
            }
        }
        switchMode($(this).val(), false);
    });
    function prepareSchedule(){
        var $enables = $('#schedule-fieldset .disableCheckbox')
        $.each($visible_textfields, function (i, $textfield) {
            var val = $textfield.val();
            if ( simple_mode && !$enables.eq(i).is(':checked') ) {
                if (val == "") {
                    val = "00:00-23:59";
                }
                val += "-disabled";
            }
            $("#weekday_" + i + "_hidden").val(val);
        });
    };
    $link_slider.change(function () {
        if ($(this).is(":checked")) {
            sliders_linked = false;
        } else {
            sliders_linked = true;
        }
    });
};
