/*global $, window, params*/
$(function () {
    "use strict";
    var $debugForm = $("#maintenance-form-record-debug-data");
    $debugForm.submit(function (e) {
        var record_time = $(this).find("#debug_record_time").val(),
            $button = $("#button-debug-download");
        e.preventDefault();
        $.ajax({
            type: "GET",
            url: "debug_download_prepare",
            cache: false,
            data: {
                record_time: record_time,
            },
            beforeSend: function () {
                $button.prop("disabled", true);
                $button.addClass("disabled");
                $button.addLoader("light");
            },
            success: function (data) {
                $button.prop("disabled", false);
                $button.removeClass("disabled");
                $button.removeLoader();
                if (data.status === "success") {
                    window.location.href = data.download_url;
                }
            },
            error: function (error) {
                console.log(error);
                $button.prop("disabled", false);
                $button.removeClass("disabled");
                $button.removeLoader();
            }
        });
    });
    $(".hasNotice").click(function (){
        var id = $(this).attr("id"),
            $parentDiv = $(this).parents(".maintenaceInner"),
            $notices = $parentDiv.find(".showOnClick.for-" + id);
        $notices.slideDown();
    });
    $("#button-restart-camera").click(function(e) {
        e.preventDefault();
        if ( confirm("This might take several minutes, do you want to continue?") ) {
            // Not tested with proxy
            $.when($.getStartTime()).done(function(ref_start_time){
                $.ajax("/axis-cgi/admin/restart.cgi");
                $.waitForApp(function(){location.reload();}, 10000, ref_start_time);
            });
        }
    });
    $("#button-restart-application").click(function(e) {
        e.preventDefault();
        if ( confirm("This might take a minute, do you want to continue?") ) {
            $.restartAndWait(function(){location.reload();});
        }
    });

    $("#button-clear-registration").click(function(e) {

        function removeLicenseKeyInApp() {
            return $.ajax({
                type: "DELETE",
                url: "/stereo/license",
                success: function() {
                    console.debug("Successfully removed license key in application.");
                },
                error: function() {
                    createErrorCallback("Could not remove license key in application");
                }
            })
        }

        function removeLicenseKeyInCamera() {
            return $.ajax({
                method: "GET",
                url: "/axis-cgi/applications/license.cgi",
                data: {
                    action: "removelicensekey",
                    package: "startapp"
                },
                success: function() {
                    console.debug("Successfully removed key in camera.");
                },
                error: function() {
                    createErrorCallback("Could not remove license key in camera.");
                }
            });
        }

            function redirect() {
                $.waitForApp(function() {
                    window.location.href = "/stereo";
                });
            }

        if ( confirm("This will remove the current registration, do you want to continue?") ) {
            removeLicenseKeyInApp().done(function() {
                removeLicenseKeyInCamera().done(function() {
                    $.restartAndWait(false)
                    .done(redirect)
                    .fail(createErrorCallback("Could not restart app."));
                });
            });
        }

        e.preventDefault();
    });

    $("#button-registration-page").click(function(e) {
        window.location.href = "/stereo/registration.html";
        e.preventDefault();
    });

    $("#button-show-logs").click(function(e) {
        window.open("/logs/stereo.log");
        e.preventDefault();
    });

    $("#button-clear-data").click(function(e) {
        if ( confirm("This will remove all historical data, do you want to continue?") ) {
            $("#overlay").show();
            $.post("/stereo/data/clear", {}, function(data) {
                if (data && data['status'] == 'CM_OK') {
                    console.log("Successfully cleared historical data");
                } else {
                    if ( data && data['message'] ) {
                        alert("Could not clear data: " + data['message']);
                    } else {
                        alert("Could not clear data.");
                    }
                }
            }).fail(function() {
                alert("Could not clear data");
            });
            /*
            / As the execution time may be so small that the overlay does not show,
            / it might seem that the button had no effect. A minor delay of half a second
            / is added so a feedback is always given to the user.
            */
            $("#overlay").delay(500).hide(0);
        }
        e.preventDefault();
    });

    $("#button-upgrade").click(function(e) {
        e.preventDefault();
        if ( !confirm("This will download and install the latest version of this application. If the device is interrupted during this process you might lose data. Do you want to continue?") ) {
            return;
        }
        $.when($.getStartTime()).done(function(ref_start_time){
            $.post('/local/startapp/cmd/upgrade', {}, function(data) {
                $('#overlay').show();
                if ( data && data.status == 'Ok' ) {
                    window.status = 'Downloading package to device...'
                    setTimeout(waitForDownload, 5000);
                } else {
                    var msg = "Could not complete upgrade.";
                    if ( data && data.status ) {
                        msg = "Could not complete upgrade: " + data.status;
                    }
                    alert(msg);
                    $.waitForApp(function(){window.location = "/stereo";}, 5000, ref_start_time);
                }

                function waitForDownload() {
                    $.get('/local/startapp/status', function(data) {

                        // No valid response from API
                        if ( !data ) {
                            alert("Could not complete upgrade.");
                            $('#overlay').hide();
                            return;
                        }
                        if (data.status == "download_failed" ) {
                            if(confirm(
                                "Could not download a new package. To solve this issue, you may need to modify your proxy settings.\n\nTo view your proxy settings, press 'Ok'. If you want to stay on this page press 'Cancel'."
                                )) {
                                window.location = "/admin/app_params.shtml?app=startapp";
                            }
                            $('#overlay').hide();
                            return;
                        }

                        // If we have landed on downloaded or install_failed, check the manifest_state
                        if (data.status == "downloaded" || data.status == "install_failed") {
                            if (data.manifest_state != "ok" && data.manifest_state != 'missing') {
                                // If manifest is not ok or missing, stop polling and identify error
                                if (data.manifest_state == 'invalid_firmware') {
                                    alert("Could not upgrade: You need to update the firmware of the device!");
                                } else if (data.manifest_state == 'invalid_startapp') {
                                    alert("Could not upgrade: You need to update your ACAP!");
                                } else if (data.manifest_state == 'invalid_startapp_and_firmware') {
                                    alert("Could not upgrade: You need to update your ACAP and the firmware on the device!");
                                } else {
                                    alert("Could not upgrade:" + data.status + ", " + data.manifest_state);
                                }
                                $('#overlay').hide();
                                return;
                            }
                        }

                        // If we are running, wait for the app and redirect
                        if (data.status == "running") {
                            alert("Your application has been upgraded. Please wait while the application is starting...");
                            $.waitForApp(function(){window.location = "/stereo";}, 5000, ref_start_time);
                            return;
                        }

                        // Intermediate states, wait for next refresh
                        setTimeout(waitForDownload, 1000);
                        return;
                    }, "json");
                }
            }, "json");
        });
    });


    function createErrorCallback(message) {
        if (typeof message === 'undefined') {
            message = "An unknown error has occurred"
        }
        var func = function() {
            console.log(message);
        }
        return func;
    }
});
