/*global $, window, params*/
$(function () {
    'use strict';
    var $form = $('#form-registration');
    var $licenseCodeField = $('#license-code');
    var $licenseCodeHolder = $licenseCodeField.parent().hide();
    var $licenseKeyField = $('#license-key');
    var $licenseKeyHolder = $licenseKeyField.parent();
    var $error = $('<div/>')
        .attr('id','registration-error')
        .addClass('displayBox')
        .addClass('error')
        .prependTo($form)
        .hide();
    var $offlineToggleButton = $('#form-registration-toggle-offline');
    var $installDemoButton = $('#form-registration-demo');
    var $installLicenseButton = $('#form-registration-submit');

    (function (d, s, id) {
        var js, fjs = d.getElementsByTagName(s)[0];
        if (d.getElementById(id)) return;
        js = d.createElement(s); js.id = id;
        js.src = 'https://www.axis.com/app/acap/sdk.js?dontcache='+$.now();
        js.onerror = function() {
            toggleOffline(true);
            $offlineToggleButton.attr('disabled', true)
        };
        js.onsuccess = function() {
            toggleOffline(false)
            $offlineToggleButton.attr('disabled', false)
        };
        fjs.parentNode.insertBefore(js, fjs);
    }(document, 'script', 'acap-jssdk'));

    function createErrorCallback(message) {
        if (typeof message === 'undefined') {
            message = "An unknown error has occurred"
        }
        var func = function() {
            $error.text(message).slideDown();
            console.debug(message);
        }
        return func;
    }

    function toggleOffline(target) {
        if (typeof target == 'undefined' ) {
            target = $licenseCodeHolder.is(':visible')
        }

        if (target) {
            $licenseCodeHolder.hide();
            $licenseKeyHolder.show();
            $offlineToggleButton.text("Register online");
        } else {
            $licenseKeyHolder.hide();
            $licenseCodeHolder.show();
            $offlineToggleButton.text("Register offline");
        }
    }

    function clearValidationErrors() {
        $('#form-registration input').each(function () {
            $(this).on('input', function () {
                $(this).removeClass('validation-error');
            });
        })
    }

    function expandVapixError(data) {
        var errorCode = data.replace(/\D/g, '');
        var message;
        switch(errorCode) {
            case '21':
                message = 'The uploaded license file contained invalid data';
                break;
            case '22':
                message = 'Failed to upload the license to the camera';
                break;
            case '24':
                message = 'Application for license is not installed';
                break;
            case '25':
                message = 'Invalid Application ID';
                break;
            case '26':
                message = 'Invalid version';
                break;
            case '30':
                message = 'Invalid Serial Number';
                break;
            case '31':
                message = 'License has expired';
                break;

            default:
                message = 'Unknown error installing license key';
                break;
        }
        return message
    }

    function normaliseLicenseCode(code) {
        return $.trim(code).replace(/[^a-zA-Z0-9]*/g,'').toUpperCase();
    }

    function installLicenseKeyInApp(licenseKey, licenseCode, licenseType) {
        function success(data) {
            console.debug("Successfully installed license key in application.")
        }

        return $.ajax({
            type: 'POST',
            url: '/stereo/license',
            data: {
                license_code: licenseCode,
                license_key: licenseKey,
                license_type: licenseType
            },
            success: success,
            error: createErrorCallback("Could not install license key in application."),
        })
    }

    function installLicenseKeyInCamera(licenseKey) {
        // We create and return our own deferred object since the axis-cgi api always returns 200.
        var deferred = $.Deferred();

        function success(data) {
            if (data == '') {
                createErrorCallback("Could not connect to activation server. Please use the 'Register offline' function.")();
                deferred.reject();
            } else if (data.indexOf('OK') != -1) {
                console.debug("Successfully installed license key in camera.")
                deferred.resolve();
            } else {
                createErrorCallback(expandVapixError(data))();
                deferred.reject();
            }
        }

        var formData = new FormData();
        var file = new Blob([licenseKey])
        formData.append('lic_file', file);

        $.ajax({
            type: 'POST',
            url: '/axis-cgi/applications/license.cgi?action=uploadlicensekey&package=' + $('#app-name').val(),
            data: formData,
            processData: false,
            contentType: false,
            timeout: 2000,
            success: success,
            error: [
                createErrorCallback('Could not upload install license key in camera.'),
                deferred.reject
                ]
        });
        return deferred
    }

    function installLicenseKey(licenseKey, licenseCode, licenseType) {
        // Use `done` method of deferred objects to ensure the key is updated in the camera *only* if the
        // application is expecting it..
        // Using functions that return a deferred object makes the code easier to read than a deep, nested
        // ajax calls.
        installLicenseKeyInApp(licenseKey, licenseCode, licenseType).done(function() {
            installLicenseKeyInCamera(licenseKey).done(function() {
                $.when($.restartAndWait(false)).done(function() {
                    window.location.href = "/stereo";
                }).fail(function(){
                    createErrorCallback("Could not restart camera.");
                    $("#overlay").hide();
                });
            });
        });
    }

    function installLicenseCode(licenseCode, licenseType) {
        console.log("installLicenseCode: '" + licenseCode + "'.");
        function success(obj) {
            if (typeof obj.error !== 'undefined') {
                $error.text(obj.error.message).slideDown()
            } else {
                installLicenseKey(obj.data.licenseKey.xml, licenseCode, licenseType)
            }
        }

        ACAP.registerLicenseKey({
            applicationId: $('#app-id').val(),
            deviceId: $('#device-id').val(),
            licenseCode: licenseCode,
        }, success);
    }

    function installLicenseDemo() {
        installLicenseCode('', "Demo")
    }

    var offlineLicenseKey = '';
    $licenseKeyField.change(function(event) {
        var reader = new FileReader();
        reader.onload = function(event) {
            offlineLicenseKey = event.target.result;
        };
        reader.readAsText(event.target.files[0]);
    });

    $offlineToggleButton.click(function(event) {
        event.preventDefault();
        event.stopPropagation();
        $error.slideUp();
        clearValidationErrors();

        toggleOffline()
    });

    $installLicenseButton.click(function(event) {
        event.preventDefault();
        event.stopPropagation();
        $error.slideUp();
        clearValidationErrors();

        if($('#license-type').val() != '') {
            if (!confirm("Your camera already has a license that will be removed. Continue anyway?")) {
                return
            }
        }

        if ($licenseCodeHolder.is(':visible')) {
            var licenseCode = normaliseLicenseCode($licenseCodeField.val())
            if (licenseCode == '') {
                $error.text("License code cannot be empty.").slideDown();
                $licenseCodeField.addClass('validation-error')
                return;
            }
            console.debug("Registering online.");
            installLicenseCode(licenseCode, "Full (Online)")
        } else {
            if (offlineLicenseKey == '') {
                $error.text("Please use 'Browse' to select a license file.").slideDown();
                $licenseKeyField.addClass('validation-error')
                return;
            }
            console.debug("Registering offline.");
            installLicenseKey(offlineLicenseKey, "N/A", "Full (offline)");
        }
    })

    $installDemoButton.click(function(event) {
        event.preventDefault();
        event.stopPropagation();
        $error.slideUp();
        clearValidationErrors();

        if($('#license-type').val() != '') {
            if (!confirm("Your camera already has a license that will be removed. Proceed anyway?")) {
                return
            }
        }

        installLicenseDemo();
    })
});
